#!/usr/bin/perl -wd

#use diagnostics;

=pod

=head1 B<NAME>

UiO::BOFH - PERL BOFH library

=head1 B<SYNOPSIS>

S<< use lib qw(/local/perl/lib/UiO); >>
E<10> E<8>
use UiO::BOFH;



=head1 B<DESCRIPTION>

Exports routines for bofh manipulation

=head1 B<EXTERNAL>

=over 4 

=item login()

=item logout()

=item run_command()

=item list_roles()

=item dhcp_assoc()

=item host_add()

=item add_role()

=item get_commands()



=back

=head1 B<INTERNAL>

This packages has no internal functions

=head1 EXAMPLES

    #!/usr/bin/perl -w
    #

    use strict;
    use UiO::BOFH;


    # BOFH login: 
    #
    my $bofh=UiO::BOFH->login($username, $passwd, $url);

    # Dump routines from bofh
    my @cmd=$bofh->get_commands();
    print Dumper @cmd;

    # Get subnet info:
    #
    my @res = $bofh->run_command("subnet_info",["129.240.176.224/29"]);

    # Add a new cf-engine role to a server
    #
    $bofh->add_role("hongset.uio.no", "without_monitoring");

    # List cf-engine roles
    my @r = $bofh->list_roles();
    print Dumper @r;

    # logout BOFH
    my $r=$bofh->logout();

=over 4 

=cut

package UiO::BOFH;

use strict;
use utf8;
use POSIX qw/strftime/;
use XMLRPC::Lite;
use Data::Dumper; 
use utf8;
binmode STDIN, ':encoding(UTF-8)';
binmode STDOUT, ':encoding(UTF-8)';

use vars qw(
		@ISA
		@EXPORT
	    );

my (
	$VERSION,
    );



use Exporter();
@ISA = qw(Exporter);

@EXPORT = qw(
        get_commands
        list_roles
        add_role
        logout
        host_info
	     );


$VERSION	= 1.01;


sub login {
    my ($self, $username, $password, $url) = @_;
    $self = {};
    $url = 'https://cerebrum-uio.uio.no:8000' if (!$url);
    $$self{'login'}= XMLRPC::Lite -> proxy($url);
    $$self{'auth'}=$$self{'login'}->call("login", $username, $password)->result();

    bless ($self);
    return $self;
}

sub logout {
    my ($self) = @_;
    my $logout = $$self{'login'}->call("logout",$$self{'auth'})->result;
    return $logout;
}

sub list_roles {
    my $self = shift;
    #my @bofh_roles = $$self{'login'}->call("run_command",$$self{'auth'},"policy_list_roles","*")->result;
    my @bofh_roles = $self->run_command("policy_list_roles", ["*"]);
    return @bofh_roles;
}


sub host_info {
    my $self = shift;
    my $hostname = shift;
    my @result = $self->run_command("host_info", [$hostname]);
    return @result;
}

sub run_command {
    my ($self, $command, @options) = @_;
    my @opts;
    foreach (@options) {
        foreach (@{$_}) {
            push (@opts, $_);
        }
    }
    my @result = $$self{'login'}->call("run_command",$$self{'auth'},$command,@opts)->result;
    return @result;
}

sub dhcp_assoc {
    my ($self, $hostname, $macAddress) = @_;
    $self->run_command("dhcp_assoc", [$hostname, $macAddress]);
}

sub host_add {
    my ($self, $hostname, $ipnet, $hinfo, $siteadmin, $comment) = @_;
    $DB::single = 1;
    use Encode;
    use Encode qw(from_to);
    from_to($comment, "utf8", "iso-8859-1");
    my $enc = Encode::find_encoding('UTF-8');
    $comment=$enc->decode($comment);
    $comment=$enc->encode($comment);

    warn ("host_add, $hostname, $ipnet, $hinfo, $siteadmin, $comment\n");
    $self->run_command("host_add", [$hostname, $ipnet, $hinfo, $siteadmin, $comment]);
}

sub add_role {
    my ($self, $hostname, $role) = @_;
    $self->run_command("host_policy_add", [$hostname, $role]);
}


sub get_commands {
    my $self = shift;
    my @commands = $$self{'login'}->call("get_commands",$$self{'auth'});
    return @commands;
}

1;

=back

=head1 SEE ALSO

L<UiO::XML> L<UiO::Utils> L<UiO::Param>

=head1 AUTHOR

Created by Ragnar Hongset @ UiO

=cut
