#!/usr/bin/perl

package UiO::Host;
use strict;
use warnings;
use Net::DNS;

use version;
our $VERSION = qv('0.0.2');

use Exporter ();
use vars qw(@ISA @EXPORT_OK @EXPORT);
@ISA = qw(Exporter);
@EXPORT_OK = qw(contact_cmd hinfo);
@EXPORT = @EXPORT_OK;

# Return absolute path of the Unix "remote shell" command that should
# be used to contact $host.  $host should be a FQDN.
sub contact_cmd {
    my $host = shift;
    my $ssh = '/usr/bin/ssh';

    if ($host =~ m{\b uio\.no \z}ixms) {
	return $ssh;
    }
    return;
}


# When called in scalar context, return the DNS HINFO information for
# $host as a single string.
#
# When called in list context, return the HINFO information for $host
# as a pair (hardware, OS).
sub hinfo {
    my $host = shift;

    my $res = Net::DNS::Resolver->new;
    my $answer = $res->search($host, 'HINFO');
    return if !defined $answer;

  RR:
    foreach my $rr ($answer->answer) {
	next RR if $rr->type ne 'HINFO';
	return wantarray ? [$rr->cpu, $rr->os]
	  : sprintf q{%s %s}, $rr->cpu, $rr->os;
    }
    return;
}

1;
