package UiO::iLO;

use version; $VERSION = qv('0.1');

require Exporter;
@UiO::iLO::ISA       = qw(Exporter);
@UiO::iLO::EXPORT    = qw(
			     ilo_major_version
			     create_xml_file
			     run_hponcfg
			     %min_firmware_version
		   );
@UiO::iLO::EXPORT_OK = ();

use strict;
use warnings;

use 5.008;

use Carp;
use Template;

## Common variables
our %min_firmware_version
  = (
     1 => 'all',
     2 => 'all',
     3 => 'all',
    );


## Find iLO version
sub ilo_major_version
{
    my $ilo_type = 0;

    open my $HPONCFG, '/sbin/hponcfg -g |'
      or croak "Couldn't run /sbin/hponcfg: $!";
    while (<$HPONCFG>) {
	$ilo_type = 3 if m{Device type = iLO 3 Driver name};
	$ilo_type = 2 if m{Device type = iLO 2 Driver name};
	$ilo_type = 1 if m{Device type = iLO Driver name};
    }
    close $HPONCFG;
    croak "(ERROR) Unknown iLO type '$ilo_type'\n" if $ilo_type !~ m{\A 1|2|3 \z}xms;
    return $ilo_type;
}


## Run template functions
sub create_xml_file {
    my $ttfile = shift;
    my $data   = shift;

    my $file = "/tmp/$ttfile";
    $file =~ s{\.tpl\z}{\.$$\.xml};
    my @searchpaths
      = ( '/usr/share/ilo2-templates' );

    # Template config
    my $config
      = {
	 INCLUDE_PATH => \@searchpaths, # or list ref
	 ABSOLUTE     => 1,             # Allow absolute file names
	 RELATIVE     => 0,             # Allow relative filenames
	};

    # Initialise new template
    my $tt = Template->new($config);

    # Template data
    my %template_data = %{ $data };

    # Process template
    $tt->process($ttfile, \%template_data, $file)
      or croak $tt->error."\n";

    return $file;
}


## Run hponcfg with options
sub run_hponcfg
{
    my $file  = shift;

    # Determine if minfwlevel needed
    my $minfwlevel  = q{};
    my $ilo_version = ilo_major_version();
    if ($min_firmware_version{$ilo_version} ne 'all') {
	$minfwlevel = "-m $min_firmware_version{$ilo_version}";
    }

    # Run hponcfg
    my $retval = system ("/sbin/hponcfg --file $file $minfwlevel");

    # Shift return value by 8, according to doc. on system()
    $retval >>= 8;

    if ($retval != 0) {
	carp "\nOOPS! Something went wrong when running hponcfg.\n";
	return 0;
    }
}
